/**
 *   Copyright 2010 Guardian News And Media 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 **/

package com.gu.openplatform.contentapi;

import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.Serializable;
import java.net.URL;
import java.util.*;

public class Content extends MutableContent implements Serializable {

    private static final Logger logger = LoggerFactory.getLogger(Content.class);

    protected Content(String id) {
        super(id);
    }

    public URL getApiUrl() {
        return apiUrl;
    }

    public String getSectionId() {
        return sectionId;
    }

    public String getSectionName() {
        return sectionName;
    }

    public String getTitle() {
        return webTitle;
    }

    public URL getWebUrl() {
        return webUrl;
    }

    public Set<String> getFieldNames() {
        return Collections.unmodifiableSet(fields.keySet());
    }

    public boolean isOfType(ContentTagType tagType) {
        for (Tag tag : allTags)
            if (tagType.getId().equals(tag.getId()))
                return true;

        return false;
    }

    public boolean hasField(String fieldName) {
        return fields.containsKey(fieldName);
    }

    public boolean hasField(FieldType fieldType) {
        return fields.containsKey(fieldType.getName());
    }

    public String getField(String fieldName) {
        if (fields.containsKey(fieldName))
            return fields.get(fieldName);

        logger.warn("Access to non-existent field: " + fieldName + " on content " + getId());
        return "";
    }

    public List<Tag> getTags() {
        return Collections.unmodifiableList(allTags);
    }

    public List<Tag> getTagsOfType(String type) {
        return getTagsOfType(TagType.typeFor(type));
    }

    public List<Tag> getTagsOfType(TagType type) {
        List<Tag> typedTags = new ArrayList<Tag>();

        for (Tag tag : allTags)
            if (tag.getType().equals(type))
                typedTags.add(tag);

        return Collections.unmodifiableList(typedTags);
    }

    public boolean hasTags() {
        return allTags.size() != 0;
    }

    public boolean hasTagsOfType(String... types) {
        TagType[] tagTypes = new TagType[types.length];

        for (int i = 0; i < types.length; i++)
            tagTypes[i] = TagType.typeFor(types[i]);

        return hasTagsOfType(tagTypes);
    }

    public boolean hasTagsOfType(TagType... types) {
        for (TagType type : types)
            if (getTagsOfType(type).isEmpty())
                return false;

        return true;
    }

    public boolean hasTag(String tagId) {
        for (Tag tag : allTags)
            if (tag.getId().equals(tagId))
                return true;

        return false;
    }

    public boolean hasTag(TagType tagType, String tagId) {
        for (Tag tag : getTagsOfType(tagType))
            if (tag.getId().equals(tagId))
                return true;

        return false;
    }

    public boolean hasTag(String tagType, String tagId) {
        return hasTag(TagType.typeFor(tagType), tagId);
    }

    public DateTime getWebPublicationDateTime() {
        return webPublicationDateTime;
    }

    public Date getWebPublicationDate() {
        return webPublicationDateTime.toDate();
    }
}
