/**
 *   Copyright 2010 Guardian News And Media 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 **/

package com.gu.openplatform.contentapi;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import static com.gu.openplatform.contentapi.TagListManipulator.addTagTypeToList;
import static java.util.Collections.unmodifiableSet;

class ContentQueryParametersMixin<T extends ApiQuery> implements ContentQueryParameters<T> {

    private final Set<TagType> tagTypesToQuery = new HashSet<TagType>();
    private final Set<FieldType> fieldsToQuery = new HashSet<FieldType>();
    private final T apiQuery;

    ContentQueryParametersMixin(T apiQuery) {
        this.apiQuery = apiQuery;
    }

    @Override
    public T showAllTagTypes() {
        tagTypesToQuery.clear();
        tagTypesToQuery.add(TagType.all);
        return apiQuery;
    }

    @Override
    public T showTagTypes(String... tagIds) {
        for (String tagId : tagIds)
            addTagTypeToList(tagTypesToQuery, TagType.typeFor(tagId));

        return apiQuery;
    }

    @Override
    public T showTagTypes(TagType... tags) {
        for (TagType tag : tags)
            addTagTypeToList(tagTypesToQuery, tag);

        return apiQuery;
    }

    @Override
    public T allFields() {
        fieldsToQuery.clear();
        fieldsToQuery.add(FieldType.showAllFields);
        return apiQuery;
    }

    @Override
    public T showField(String... fieldNames) {
        for (String fieldName : fieldNames)
            fieldsToQuery.add(FieldType.typeFor(fieldName));

        return apiQuery;
    }

    @Override
    public T showField(FieldType... fieldTypes) {
        fieldsToQuery.addAll(Arrays.asList(fieldTypes));
        return apiQuery;
    }

    Set<TagType> getTagTypesToFind() {
        return unmodifiableSet(tagTypesToQuery);
    }

    boolean isRetrievingAllFields() {
        return fieldsToQuery.contains(FieldType.showAllFields);
    }

    Set<? extends ApiUrlParameter> getFieldsToFind() {
        return unmodifiableSet(fieldsToQuery);
    }

    void fillInApiParameters(Map<ApiParameter, Object> parameters) {
        apiQuery.addHttpParameter(parameters, ApiParameter.showFields, fieldsToQuery);
        apiQuery.addHttpParameter(parameters, ApiParameter.showTags, tagTypesToQuery);
    }
}

