/**
 *   Copyright 2010 Guardian News And Media
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 **/

package com.gu.openplatform.contentapi;

import org.joda.time.DateTime;

import java.util.Date;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import static com.gu.openplatform.contentapi.TagListManipulator.addTagTypeToList;
import static java.util.Collections.unmodifiableSet;

public class ContentSearchQuery extends PaginatedSearchQuery<ContentSearchQuery> {

    private final Set<StringUrlParameter> tagsToFilter = new HashSet<StringUrlParameter>();
    private final Set<TagType> refinementsToReturn = new HashSet<TagType>();
    private final ContentQueryParametersMixin<ContentSearchQuery> contentQueryParameters = new ContentQueryParametersMixin<ContentSearchQuery>(this);

    private DateTime fromDate;
    private DateTime toDate;

    public ContentSearchQuery(SearchJsonParser jsonParser, HttpClient httpClient) {
        super(httpClient, jsonParser);
    }

    public ContentSearchResult execute() {
        ContentSearchResult result = new ContentSearchResult(this);

        try {
            jsonParser.parseJson(httpClient.executeQuery("/search", this), result);
        }
        catch (HttpException e) {
            result.setStatus(ResponseStatus.error);
        }

        return result;
    }

    public ContentSearchQuery showRefinementsOfType(TagType tagType) {
        addTagTypeToList(refinementsToReturn, tagType);
        return ContentSearchQuery.this;
    }

    public ContentQueryParameters<ContentSearchQuery> showRefinementsOfType(TagType... types) {
        for (TagType type : types)
            addTagTypeToList(refinementsToReturn, type);

        return contentQueryParameters;
    }

    public ContentSearchQuery showRefinementsOfType(String... types) {
        for (String type : types)
            showRefinementsOfType(TagType.typeFor(type));

        return ContentSearchQuery.this;
    }

    public ContentQueryParameters<ContentSearchQuery> showAllRefinements() {
        refinementsToReturn.clear();
        showRefinementsOfType(TagType.all);
        return contentQueryParameters;
    }

    public ContentSearchQuery filterByTag(Tag tag) {
        tagsToFilter.add(new StringUrlParameter(tag.getId()));
        return ContentSearchQuery.this;
    }

    public ContentQueryParameters<ContentSearchQuery> filterByTag(Tag... tags) {
        for (Tag tag : tags)
            tagsToFilter.add(new StringUrlParameter(tag.getId()));

        return contentQueryParameters;
    }

    public ContentSearchQuery filterByTag(String... tagIds) {
        for (String tagId : tagIds)
            tagsToFilter.add(new StringUrlParameter(tagId));

        return ContentSearchQuery.this;
    }

    public ContentSearchQuery filterByTag(String tagId) {
        tagsToFilter.add(new StringUrlParameter(tagId));
        return ContentSearchQuery.this;
    }

    @Override
    protected void fillInApiParameters(Map<ApiParameter, Object> parameters) {
        super.fillInApiParameters(parameters);
        contentQueryParameters.fillInApiParameters(parameters);

        addHttpParameter(parameters, ApiParameter.tag, tagsToFilter);
        addHttpParameter(parameters, ApiParameter.showRefinements, refinementsToReturn);
        addHttpParameter(parameters, ApiParameter.fromDate, fromDate);
        addHttpParameter(parameters, ApiParameter.toDate, toDate);
    }

    public ContentSearchQuery showAllTagTypes() {
        return contentQueryParameters.showAllTagTypes();
    }

    public ContentSearchQuery showTagTypes(String... tagIds) {
        return contentQueryParameters.showTagTypes(tagIds);
    }

    public ContentSearchQuery showTagTypes(TagType... tags) {
        return contentQueryParameters.showTagTypes(tags);
    }

    public ContentSearchQuery allFields() {
        return contentQueryParameters.allFields();
    }

    public ContentSearchQuery showField(String... fieldNames) {
        return contentQueryParameters.showField(fieldNames);
    }

    public ContentSearchQuery showField(FieldType... fieldTypes) {
        return contentQueryParameters.showField(fieldTypes);
    }

    Set<TagType> getRefinementsToReturn() {
        return unmodifiableSet(refinementsToReturn);
    }

    boolean isRetrievingAllFields() {
        return contentQueryParameters.isRetrievingAllFields();
    }

    Set<? extends ApiUrlParameter> getFieldsToFind() {
        return contentQueryParameters.getFieldsToFind();
    }

    Set<TagType> getTagTypesToFind() {
        return contentQueryParameters.getTagTypesToFind();
    }

    public ContentSearchQuery fromDate(DateTime fromDate) {
        this.fromDate = fromDate;
        return this;
    }

    public ContentSearchQuery fromDate(Date fromDate) {
        this.fromDate = new DateTime(fromDate);
        return this;
    }

    DateTime getFromDate() {
        return fromDate;
    }

    public ContentSearchQuery toDate(DateTime toDate) {
        this.toDate = toDate;
        return this;
    }

    public ContentSearchQuery toDate(Date toDate) {
        this.toDate = new DateTime(toDate);
        return this;
    }

    DateTime getToDate() {
        return toDate;
    }

}
