/**
 *   Copyright 2010 Guardian News And Media
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 **/

package com.gu.openplatform.contentapi;

import java.util.*;

import static java.util.Collections.unmodifiableList;
import static java.util.Collections.unmodifiableMap;

public class ContentSearchResult extends PaginatedSearchResult<ContentSearchQuery> {

    private final List<Content> content = new ArrayList<Content>();

    private Map<TagType, List<Refinement>> refinements;
    private List<Refinement> allRefinements;
    private OrderBy orderBy;

    protected ContentSearchResult(ContentSearchQuery query) {
        super(query);
    }

    public OrderBy getOrderBy() {
        return orderBy;
    }

    public List<Content> getContent() {
        return unmodifiableList(content);
    }

    public boolean hasContent() {
        return isResponseOk() && !content.isEmpty();
    }

    public Content getFirst() {
        return content.get(0);
    }

    public boolean hasRefinements() {
        return !(refinements == null);
    }

    public Map<TagType, List<Refinement>> getRefinements() {
        return refinements == null ? Collections.<TagType, List<Refinement>>emptyMap() : unmodifiableMap(refinements);
    }

    public List<Refinement> getRefinements(TagType tagType) {
        List<Refinement> refinements = getRefinements().get(tagType);

        if (refinements == null)
            return Collections.emptyList();

        return refinements;
    }

    public boolean hasRefinements(TagType tagType) {
        return getRefinements().get(tagType) != null;
    }

    public List<Refinement> getAllRefinements() {
        return allRefinements == null ? Collections.<Refinement>emptyList() : unmodifiableList(allRefinements);
    }

    void addRefinement(Refinement refinement) {
        if (refinements == null) {
            allRefinements = new ArrayList<Refinement>();
            refinements = new HashMap<TagType, List<Refinement>>();
        }

        TagType type = refinement.getType();
        List<Refinement> refinementsOfType = refinements.get(type);

        if (refinementsOfType == null) {
            refinementsOfType = new ArrayList<Refinement>();
            refinements.put(type, refinementsOfType);
        }

        refinementsOfType.add(refinement);
        allRefinements.add(refinement);
    }

    void addContent(Content content) {
        this.content.add(content);
    }

    void setOrderBy(OrderBy orderBy) {
        this.orderBy = orderBy;
    }


}
