/**
 *   Copyright 2010 Guardian News And Media 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 **/

package com.gu.openplatform.contentapi;

import com.gu.openplatform.contentapi.configuration.ConfigurationProfile;
import org.picocontainer.DefaultPicoContainer;
import org.picocontainer.MutablePicoContainer;
import org.picocontainer.behaviors.OptInCaching;

import java.net.URL;

import static org.picocontainer.Characteristics.CACHE;
import static org.picocontainer.Characteristics.NO_CACHE;

public abstract class GuardianContentApi {

    private static final MutablePicoContainer container;

    static {
        container = new DefaultPicoContainer(new OptInCaching());

        container.as(NO_CACHE).addComponent(ContentSearchQuery.class);
        container.as(NO_CACHE).addComponent(TagSearchQuery.class);
        container.as(NO_CACHE).addComponent(ItemQuery.class);

        container.as(CACHE).addComponent(ItemJsonParser.class);
        container.as(CACHE).addComponent(HttpClient.class);
        container.as(CACHE).addComponent(HttpQueryExecutor.class);
        container.as(CACHE).addComponent(SearchJsonParser.class);
        container.as(CACHE).addComponent(ApiUrlFactory.class);
        container.as(CACHE).addComponent(TagJsonParser.class);

        container.start();
    }

    /**
     * @return client configured to retrieve the minimum content information, no fields, tags or refinements
     */

    public static ContentSearchQuery minimalContentSearchQuery() {
        return container.getComponent(ContentSearchQuery.class);
    }

    /**
     * @return client configured to retrieve all content tags & fields. No refinements will be returned on the q
     */

    public static ContentSearchQuery contentSearchQuery() {
        return container
                .getComponent(ContentSearchQuery.class)
                .configure(ConfigurationProfile.allTagsAndFields);
    }

    /**
     * @return client configured to retrieve all content tags, fields & refiements.
     */

    public static ContentSearchQuery contentSearchQueryWithRefinements() {
        return container
                .getComponent(ContentSearchQuery.class)
                .configure(ConfigurationProfile.allTagsAndFieldsAndRefinements);
    }

    public static TagSearchQuery tagSearchQuery() {
        return container.getComponent(TagSearchQuery.class);
    }

    public static ItemQuery itemQuery(String apiUrl) {
        return itemQuery().url(apiUrl);
    }

    public static ItemQuery itemQuery() {
        return container
                .getComponent(ItemQuery.class)
                .configure(ConfigurationProfile.allItemTagsAndFields);
    }

    public static ItemQuery itemQuery(URL apiUrl) {
        return itemQuery().configure(ConfigurationProfile.allItemTagsAndFields).url(apiUrl);
    }
}
