/**
 *   Copyright 2010 Guardian News And Media 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 **/

package com.gu.openplatform.contentapi;

import net.sf.json.JSONArray;
import net.sf.json.JSONException;
import net.sf.json.JSONObject;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;

abstract class JsonParser {

    private static final Logger logger = LoggerFactory.getLogger(JsonParser.class);

    void parseJson(String json, ApiResponse result) {
        result.setJson(json);
    }

    protected URL convertToUrl(String path) {
        URL result;

        try {
            result = new URL(path);
        }
        catch (MalformedURLException e) {
            throw new JSONException("Could not parse URL", e);
        }

        return result;
    }

    protected Tag parseTag(JSONObject jsonObject) {
        if (jsonObject.has("sectionId"))
            return new Tag(
                    jsonObject.getString("id"),
                    TagType.typeFor(jsonObject.getString("type")),
                    jsonObject.getString("webTitle"),
                    jsonObject.getString("webUrl"),
                    jsonObject.getString("apiUrl"),
                    jsonObject.getString("sectionName"),
                    jsonObject.getString("sectionId"));
        else
            return new Tag(
                    jsonObject.getString("id"),
                    TagType.typeFor(jsonObject.getString("type")),
                    jsonObject.getString("webTitle"),
                    jsonObject.getString("webUrl"),
                    jsonObject.getString("apiUrl"));
    }

    protected Content parseContent(JSONObject jsonContent) {
        Content content = new Content(jsonContent.getString("id"));

        content.setSectionId(jsonContent.getString("sectionId"));
        content.setSectionName(jsonContent.getString("sectionName"));
        content.setApiUrl(convertToUrl(jsonContent.getString("apiUrl")));
        content.setWebUrl(convertToUrl(jsonContent.getString("webUrl")));
        content.setWebTitle(jsonContent.getString("webTitle"));
        content.setWebPublicationDate(new DateTime(jsonContent.getString("webPublicationDate")));

        if (jsonContent.has("fields"))
            parseContentFields(content, jsonContent.getJSONObject("fields"));

        if (jsonContent.has("tags"))
            parseContentTags(content, jsonContent.getJSONArray("tags"));

        return content;
    }

    protected <T extends ApiResponse> JSONObject extractTopLevelResponse(T searchResponse, String json) {
        JSONObject jsonObject = parseCoreResponseJson(json);
        populateCoreResponseFields(searchResponse, jsonObject);
        return jsonObject;
    }

    protected void populateCoreResponseFields(ApiResponse apiResponse, JSONObject jsonResponse) {
        apiResponse.setStatus(ResponseStatus.valueOf(jsonResponse.getString("status")));
        apiResponse.setUserTier(UserTier.valueOf(jsonResponse.getString("userTier")));
        apiResponse.setTotalNumberOfResults(jsonResponse.getInt("total"));
    }

    protected JSONObject parseCoreResponseJson(String json) {
        JSONObject result;

        try {
            result = JSONObject.fromString(json).getJSONObject("response");
        }
        catch (JSONException e) {
            logger.error("Could not parse json: ", e);
            throw e;
        }

        return result;
    }

    @SuppressWarnings("unchecked")
    private void parseContentTags(Content content, JSONArray jsonObject) {
        Iterator<JSONObject> jsonTags = jsonObject.iterator();

        while (jsonTags.hasNext())
            content.addTag(parseTag(jsonTags.next()));
    }

    @SuppressWarnings("unchecked")
    private void parseContentFields(Content content, JSONObject jsonFields) {
        Iterator<String> fields = jsonFields.keys();

        while (fields.hasNext()) {
            String fieldName = fields.next();
            content.addField(fieldName, jsonFields.getString(fieldName));
        }
    }
}
