/**
 *   Copyright 2010 Guardian News And Media 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 **/

package com.gu.example;

import com.gu.openplatform.contentapi.*;
import org.junit.After;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.greaterThan;
import static org.junit.Assert.*;

public class ExampleTagSearchTest {

    private static final Logger logger = LoggerFactory.getLogger(ExampleTagSearchTest.class);

    private TagSearchResult results;

    @After
    public void tearDown() throws Exception {
        checkResponseOK();
    }

    private void checkResponseOK() {
        if (!results.isResponseOk())
            logger.error("Invalid response recieved from API");

        assertTrue(results.isResponseOk());
        assertTrue(results.hasTags());
    }

    @Test
    public void testCanSearchForTags() throws Exception {
        results = GuardianContentApi.tagSearchQuery()
                .zeroTermQuery()
                .execute();

        checkResponseOK();
        assertTrue(results.getCurrentPage() == 1);
        assertTrue(results.hasTags());

        for (Tag tag : results.getTags())
            TagValidator.validateTag(tag);
    }

    @Test
    public void testCanRetrieveOriginalQueryFromResponse() throws Exception {
        TagSearchQuery query = GuardianContentApi.tagSearchQuery().zeroTermQuery();
        results = query.execute();

        checkResponseOK();
        assertThat((TagSearchQuery) results.getQuery(), equalTo(query));
    }

    @Test
    public void testCanPaginateResultSet() throws Exception {
        results = GuardianContentApi.tagSearchQuery().execute();

        checkResponseOK();
        assertThat(results.getNumberOfPages(), greaterThan(1));

        int foundPages = 1;
        for (TagSearchQuery nextPageQuery : results.getResultPageQueries())
            foundPages = nextPageQuery.getStartPage();

        assertThat(results.getNumberOfPages(), equalTo(foundPages));
    }

    @Test
    public void testCanJumpToPageInResultSet() throws Exception {
        results = GuardianContentApi.tagSearchQuery().execute();

        checkResponseOK();
        assertThat(results.getNumberOfPages(), greaterThan(5));

        TagSearchQuery page5Query = results.getPageQuery(5);
        assertThat(page5Query.getStartPage(), equalTo(5));

        try {
            results.getPageQuery(results.getNumberOfPages() + 1);
            fail("Should not be able to get pages that don't exist");
        }
        catch (IllegalStateException e) {
            //no-op
        }
    }

    @Test
    public void testPageSizeCorrect() throws Exception {
        results = GuardianContentApi.tagSearchQuery()
                .pageSize(30)
                .zeroTermQuery()
                .execute();

        checkResponseOK();
        assertThat(results.getPageSize(), equalTo(30));
        assertThat(results.getCurrentPage(), equalTo(1));
    }

    @Test
    public void testCanFilterTagsByType() throws Exception {
        results = GuardianContentApi.tagSearchQuery()
                .type(TagType.series)
                .execute();

        checkResponseOK();
        assertTrue(results.hasTags());

        for (Tag tag : results.getTags())
            assertTrue(tag.getType().equals(TagType.series));

        results = GuardianContentApi.tagSearchQuery()
                .type("keyword")
                .execute();

        assertTrue(results.hasTags());

        for (Tag tag : results.getTags())
            assertThat(tag.getType(), equalTo(TagType.keyword));
    }

    @Test
    public void testCanFilterAllTagTypes() throws Exception {
        for (TagType type : TagType.allTypes) {
            results = GuardianContentApi.tagSearchQuery()
                    .type(type)
                    .execute();

            checkResponseOK();
            assertTrue(results.hasTags());

            for (Tag tag : results.getTags())
                assertThat(tag.getType(), equalTo(type));
        }
    }

    @Test
    public void testCanFilterTagsByMultipleTypes() throws Exception {
        results = GuardianContentApi.tagSearchQuery()
                .type("blog", "keyword")
                .execute();

        checkResponseOK();
        assertTrue(results.hasTags());

        for (Tag tag : results.getTags())
            assertTrue(tag.isOfType(TagType.keyword) || tag.isOfType("blog"));
    }

    @Test
    public void testCanFilterTagsBySection() throws Exception {
        results = GuardianContentApi.tagSearchQuery()
                .section("business")
                .execute();

        checkResponseOK();
        assertTrue(results.hasTags());

        for (Tag tag : results.getTags())
            if (tag.hasSection())
                assertTrue(tag.getSectionId().equals("business"));
    }
}
