/**
 *   Copyright 2010 Guardian News And Media 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 **/

package com.gu.openplatform.contentapi;

import com.gu.openplatform.contentapi.configuration.ConfigurationProfile;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.Mockito;

import java.io.IOException;
import java.net.URL;

import static org.mockito.Mockito.*;

public class ContentSearchQueryHttpParamsTest extends HttpParamsTest {

    private ContentSearchQuery contentSearchQuery;
    @Mock private SearchJsonParser jsonParser;

    @Before
    public void setUp() throws Exception {
        contentSearchQuery = new ContentSearchQuery(jsonParser, new HttpClient(new ApiUrlFactory(), httpQueryExecutor));
        contentSearchQuery.configure(ConfigurationProfile.allTagsAndFields);
    }

    @Test
    public void defaultBehaviourForQueriesIsCorrect() throws Exception {
        contentSearchQuery.execute();

        verify(httpQueryExecutor).execute(argThat(expectParameters(
                new HttpParam(ApiParameter.showTags, "all"),
                new HttpParam(ApiParameter.format, "json"),
                new HttpParam(ApiParameter.showFields, "all"),
                new HttpParam(ApiParameter.pageSize, "10"))));
    }

    @Test
    public void testCorrectlyDetectErrorsFromHttpService() throws Exception {
        when(httpQueryExecutor.execute(Mockito.<URL>any())).thenThrow(new IOException());

        contentSearchQuery.execute();
    }

    @Test
    public void testQueryParameterIsCorrectlyPassedThrough() throws Exception {
        contentSearchQuery.query("sausages").execute();

        verify(httpQueryExecutor).execute(argThat(expectParameters(
                new HttpParam(ApiParameter.q, "sausages"))));
    }


    @Test
    public void removesShowAllTagsWhenSpecificTagTypeIsRequested() throws Exception {
        contentSearchQuery.showTagTypes(TagType.keyword).execute();

        verify(httpQueryExecutor).execute(argThat(expectParameters(new HttpParam(ApiParameter.showTags, "keyword"))));
    }

    @Test
    public void testAppliesTagIdFilter() throws Exception {
        contentSearchQuery.filterByTag("world/news", "fish/chips").execute();

        verify(httpQueryExecutor).execute(argThat(expectParameters(new HttpParam(ApiParameter.tag, "world/news,fish/chips"))));
    }

    @Test
    public void testRemovesAllTagsWhenSpecificTagTypeIsAdded() throws Exception {
        contentSearchQuery.showTagTypes(TagType.keyword).execute();

        verify(httpQueryExecutor).execute(argThat(expectParameters(new HttpParam(ApiParameter.showTags, "keyword"))));
    }

    @Test
    public void testPassesPageSizeParameterToHttpClient() throws Exception {
        contentSearchQuery.pageSize(20).execute();

        verify(httpQueryExecutor).execute(argThat(expectParameters(new HttpParam(ApiParameter.pageSize, "20"))));
    }

    @Test
    public void testPassesApiKeyParameterToHttpClient() throws Exception {
        contentSearchQuery.apiKey("test").execute();

        verify(httpQueryExecutor).execute(argThat(expectParameters(new HttpParam(ApiParameter.apiKey, "test"))));
    }

    @Test
    public void testPassesQueryParameterToHttpClient() throws Exception {
        contentSearchQuery.query("sausages").execute();

        verify(httpQueryExecutor).execute(argThat(expectParameters(new HttpParam(ApiParameter.q, "sausages"))));
    }

    @Test
    public void asksForTypeTagsByDefault() throws Exception {
        contentSearchQuery.showTagTypes(TagType.all).execute();

        verify(httpQueryExecutor).execute(argThat(expectParameters(new HttpParam(ApiParameter.showTags, TagType.all))));
    }

    @Test
    public void testComplexQuery() throws Exception {
        contentSearchQuery
                .apiKey("testapi")
                .query("foo")
                .pageSize(20)
                .showTagTypes(TagType.keyword, TagType.contributor)
                .execute();

        verify(httpQueryExecutor).execute(argThat(expectParameters(
                new HttpParam(ApiParameter.pageSize, "20"),
                new HttpParam(ApiParameter.apiKey, "testapi"),
                new HttpParam(ApiParameter.showTags, "keyword,contributor"),
                new HttpParam(ApiParameter.q, "foo"))));
    }
}
