/**
 *   Copyright 2010 Guardian News And Media 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 **/

package com.gu.openplatform.contentapi;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import static com.gu.openplatform.contentapi.ClasspathFileLoader.loadFileContents;
import static com.gu.openplatform.contentapi.ContentValidator.validateContent;
import static com.gu.openplatform.contentapi.TagValidator.validateTag;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.notNullValue;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class JsonParserTest {

    private static final DateTime expectedDateTime = new DateTime("2010-04-06T11:00:00+01:00");

    private SearchJsonParser searchJsonParser;
    private ContentSearchResult result;

    @Before
    public void setUp() throws Exception {
        searchJsonParser = new SearchJsonParser();
        result = new ContentSearchResult(GuardianContentApi.minimalContentSearchQuery());
    }

    @Test
    public void testJsonParserCanExtractCoreResponseFields() throws Exception {
        searchJsonParser.parseJson(loadFileContents("basic-ok-search.json"), result);

        assertThat(result.getStatus(), equalTo(ResponseStatus.ok));
        assertThat(result.getUserTier(), equalTo(UserTier.free));
        assertThat(result.getTotalNumberOfResults(), equalTo(1159612));
        assertThat(result.getStartIndex(), equalTo(1));
        assertThat(result.getPageSize(), equalTo(10));
        assertThat(result.getCurrentPage(), equalTo(2));
        assertThat(result.getNumberOfPages(), equalTo(115962));
        assertThat(result.getOrderBy(), equalTo(OrderBy.newest));
    }

    @Test
    public void testJsonParserCanExtractAllContentInformation() throws Exception {
        searchJsonParser.parseJson(loadFileContents("basic-ok-search-with-fields-and-tags.json"), result);

        assertThat(result.hasContent(), equalTo(true));
        assertThat(result.getContent().size(), equalTo(2));
        Iterator<Content> contentIter = result.getContent().iterator();

        for (Integer expected : Arrays.asList(1, 2))
            runTestsOnContent(contentIter.next(), expected);
    }

    @Test
    public void testCanExtractContentFields() throws Exception {
        searchJsonParser.parseJson(loadFileContents("basic-ok-search-with-content-fields.json"), result);
        Iterator<Content> contentIter = result.getContent().iterator();

        for (Integer expected : Arrays.asList(1, 2))
            checkContentFieldsMatchExpected(expected, contentIter.next());
    }

    private void checkContentFieldsMatchExpected(Integer expected, Content content) {
        Set<String> fields = content.getFieldNames();
        assertThat(fields.size(), equalTo(2));

        for (String field : fields)
            assertThat(field, notNullValue());

        assertThat(content.getField("headline"), equalTo(expected.toString()));
    }

    private void runTestsOnContent(Content content, Integer expected) {
        validateContent(content);
        checkContentDateMatchesExpected(content);
        checkContentIsWellFormed(content, expected);
        assertFalse(content.getTags().isEmpty());
        checkContentContainsAtLeastOneWellFormedTagOfEachType(content);
    }

    private void checkContentDateMatchesExpected(Content content) {
        assertThat(content.getWebPublicationDateTime(), equalTo(expectedDateTime));
        assertThat(content.getWebPublicationDate(), equalTo(expectedDateTime.toDate()));
    }

    private void checkContentContainsAtLeastOneWellFormedTagOfEachType(Content content) {
        for (TagType tagType : TagType.allTypes) {
            List<Tag> tags = content.getTagsOfType(tagType);

            assertFalse("Expecting tag of type " + tagType.asApiUrlParameter(), tags.isEmpty());
            validateTag(tags.iterator().next());
        }
    }

    private void checkContentIsWellFormed(Content content, Integer expected) {
        assertThat(content.getApiUrl().toString(), equalTo("http://content.guardianapis.com/" + expected));
        assertThat(content.getWebUrl().toString(), equalTo("http://content.guardianapis.com/" + expected));
        assertThat(content.getId(), equalTo(expected.toString()));
        assertThat(content.getSectionId(), equalTo(expected.toString()));
        assertThat(content.getSectionName(), equalTo(expected.toString()));
        assertThat(content.getTitle(), equalTo(expected.toString()));
        assertTrue(content.hasField("headline"));
        assertTrue(content.hasField("byline"));
        assertFalse(content.hasField("body"));
    }


}
