/**
 *   Copyright 2010 Guardian News And Media
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 **/

package com.gu.openplatform.contentapi;

import com.gu.openplatform.contentapi.configuration.ConfigurationProfile;
import com.gu.openplatform.contentapi.configuration.search.RetrieveAllFieldsConfiguration;
import com.gu.openplatform.contentapi.configuration.search.ShowAllTagsConfiguration;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class SearchClientConfigurationTest {

    private ContentSearchQuery contentSearchQuery;
    @Mock private SearchJsonParser searchJsonParser;
    @Mock private HttpClient httpClient;

    @Before
    public void setUp() throws Exception {
        contentSearchQuery = new ContentSearchQuery(searchJsonParser, httpClient);
    }

    @Test
    public void testCanConfigureRefinements() throws Exception {
        assertThat(contentSearchQuery.getRefinementsToReturn().size(), equalTo(0));

        contentSearchQuery.showRefinementsOfType(TagType.keyword);

        assertThat(contentSearchQuery.getRefinementsToReturn().size(), equalTo(1));
        assertTrue(contentSearchQuery.getRefinementsToReturn().contains(TagType.keyword));
    }

    @Test
    public void testAddingRefinementRemovesShowAllRefinements() throws Exception {
        contentSearchQuery.showAllRefinements();
        contentSearchQuery.showRefinementsOfType(TagType.keyword);

        assertFalse(contentSearchQuery.getRefinementsToReturn().contains(TagType.all));
        assertTrue(contentSearchQuery.getRefinementsToReturn().contains(TagType.keyword));
    }

    @Test
    public void testCanConfigureMultipleRefinements() throws Exception {
        contentSearchQuery.showRefinementsOfType(TagType.keyword, TagType.blog);

        assertThat(contentSearchQuery.getRefinementsToReturn().size(), equalTo(2));
        assertTrue(contentSearchQuery.getRefinementsToReturn().contains(TagType.keyword));
        assertTrue(contentSearchQuery.getRefinementsToReturn().contains(TagType.blog));
    }

    @Test
    public void testCanConfigureRefinementsWithTextualNames() throws Exception {
        contentSearchQuery.showRefinementsOfType("keyword", "blog");

        assertThat(contentSearchQuery.getRefinementsToReturn().size(), equalTo(2));
        assertTrue(contentSearchQuery.getRefinementsToReturn().contains(TagType.keyword));
        assertTrue(contentSearchQuery.getRefinementsToReturn().contains(TagType.blog));
    }

    @Test
    public void testCanCongfigureAllFields() throws Exception {
        assertFalse(contentSearchQuery.isRetrievingAllFields());
        contentSearchQuery.configure(new RetrieveAllFieldsConfiguration());

        assertTrue(contentSearchQuery.isRetrievingAllFields());
        assertTrue(contentSearchQuery.getFieldsToFind().size() == 1);
        assertTrue(contentSearchQuery.getFieldsToFind().contains(FieldType.showAllFields));
    }

    @Test
    public void testCanApplyConfiguration() throws Exception {
        contentSearchQuery.configure(ConfigurationProfile.allTagsAndFields);
        assertThat(contentSearchQuery.getPageSize(), equalTo(10));
        assertTrue(contentSearchQuery.getTagTypesToFind().contains(TagType.all));
    }

    @Test
    public void testAddingContentTypeToConfigurationRemovesTypeOfAllIfPresent() throws Exception {
        contentSearchQuery.configure(new ShowAllTagsConfiguration());
        contentSearchQuery.showTagTypes(TagType.keyword);

        assertThat(contentSearchQuery.getTagTypesToFind().size(), equalTo(1));
        assertTrue(contentSearchQuery.getTagTypesToFind().contains(TagType.keyword));
    }

    @Test
    public void canConfigureTags() throws Exception {
        assertThat(contentSearchQuery.getTagTypesToFind().size(), equalTo(0));

        contentSearchQuery.showTagTypes("foo");

        assertThat(contentSearchQuery.getTagTypesToFind().size(), equalTo(1));
        assertTrue(contentSearchQuery.getTagTypesToFind().contains(TagType.typeFor("foo")));
    }

    @Test
    public void canCreateContentApiClientWithApiKey() throws Exception {
        contentSearchQuery.apiKey("testapikey");
        assertThat(contentSearchQuery.getApiKey(), equalTo("testapikey"));
    }

    @Test
    public void canConfigurePageSize() throws Exception {
        assertThat(contentSearchQuery.getPageSize(), equalTo(null));

        checkPageSize(1);
        checkPageSize(30);
        checkPageSize(50);

        try {
            checkPageSize(51);
            fail("Should not be able to set page size > 50");
        }
        catch (IllegalArgumentException e) {
            // ok
        }

        try {
            checkPageSize(0);
            fail("Shoult not be able to set page size < 1");
        }
        catch (IllegalArgumentException e) {
            // ok
        }
    }

    private void checkPageSize(Integer pageSize) {
        contentSearchQuery.pageSize(pageSize);
        assertThat(contentSearchQuery.getPageSize(), equalTo(pageSize));
    }

    @Test
    public void canCreateContentApiClientWithoutApiKey() throws Exception {
        assertThat(contentSearchQuery.getApiKey(), equalTo(null));
    }

    @Test
    public void canConfigureSearchTerms() throws Exception {
        contentSearchQuery.query("foo");
        assertThat(contentSearchQuery.getQueryString(), equalTo("foo"));

        contentSearchQuery = GuardianContentApi.contentSearchQuery().zeroTermQuery();
        assertThat(contentSearchQuery.getQueryString(), equalTo(null));

        contentSearchQuery = GuardianContentApi.contentSearchQuery();
        assertThat(contentSearchQuery.getQueryString(), equalTo(null));
    }
}
